package;

import haxegon.*;

class Bug extends Entity
{
	public var angle:Float;

	public var anim:String;
	public var animframe:Int;

	// flag set when destroyed so we can clean up properly
	public var killed:Bool;
	
	// AI stuff
	public var target:Int;
	public var targetx:Float;
	public var targety:Float;
	var favoursleft:Bool;
	
	public var infesting:Bool;
	public var infesttimer:Int;
	
	var exploringframes:Int;
	var explorex:Float;
	var explorey:Float;
	
	var rundesire:Int;
	
	private function setanim(_newanim:String)
	{
		if (_newanim != anim) {
			anim = _newanim;
			animframe = 0;
		}
	}
	
	public function new(_x:Float, _y:Float, _angle:Float)
	{
		super();
		
		x = _x;
		y = _y;
		
		angle = _angle * Math.PI / 180;
		killed = false;
		
		rundesire = 30;
		favoursleft = Random.bool();
		
		setanim("idle");
		
		if (Random.chance(25) && GameSequence.allowbugtargetplayer)
		{
			// 1 in 4 bugs chase the player
			target = Hardware.PLAYER;
		}
		else
		{
			var targets:Array<Int> = [];
			
			targets.push(Hardware.REACTOR);
			targets.push(Hardware.REACTOR);
			targets.push(Hardware.REACTOR);
			targets.push(Hardware.REACTOR);
			targets.push(Hardware.REACTOR);
			targets.push(Hardware.REACTOR);
			
			if (GameSequence.allowbugtargetbuttons) {
				targets.push(Hardware.X);
				targets.push(Hardware.X);
				targets.push(Hardware.Z);
				targets.push(Hardware.Z);
			}
			if (GameSequence.allowbugtargetdpad) {
				targets.push(Hardware.DPAD_UP);
				targets.push(Hardware.DPAD_DOWN);
				targets.push(Hardware.DPAD_LEFT);
				targets.push(Hardware.DPAD_RIGHT);
			}
			if (GameSequence.allowbugtargetscreen) {
				targets.push(Hardware.SCREEN);
				targets.push(Hardware.SCREEN);
				targets.push(Hardware.SCREEN);
				targets.push(Hardware.SCREEN);
				targets.push(Hardware.SCREEN);
			}
				
			if (targets.length > 0) {
				target = targets[Random.int(0, targets.length - 1)];
			}
		}
		
		targetx = Hardware.gettargetx(target);
		targety = Hardware.gettargety(target);
	}

	public override function update()
	{
		++animframe;
		
		// Are we dead?
		if (killed) {
			Main.get().internal_entities.remove(this);
			
			Sound.play("kill");
			
			if (infesting) {
				Hardware.removebug(target);
			}
			return;
		}
		
		// Work out utility vectors.
		var forwardx:Float = Math.sin(angle);
		var forwardy:Float = -Math.cos(angle);

		var rightx:Float = Math.cos(angle);
		var righty:Float = Math.sin(angle);

		// Where are we headed?
		if (target == Hardware.PLAYER) {
			targetx = Hardware.gettargetx(target);
			targety = Hardware.gettargety(target);
		}
		
		var targetdirx:Float = targetx - x;
		var targetdiry:Float = targety - y;
		var targetdist:Float = Math.sqrt(targetdirx * targetdirx + targetdiry * targetdiry);

		if (infesting) {
			infesttimer--;
			
			if (infesttimer <= 0) {
				Hardware.addinfestation(target, this);
				infesttimer = Random.int(80, 150);
			}
			return;
		}
		
		if (targetdist < 6 && target != Hardware.PLAYER) {
			infesting = true;
			
			var targetgoalangle:Float = Math.atan2(targetdirx, -targetdiry);
			angle = targetgoalangle;
			
			setanim("infest");
			
			Hardware.addbug(target);
			
			infesttimer = Random.int(50, 100);
			
			return;
		}
		
		var navgoalx:Float = targetx;
		var navgoaly:Float = targety;

		// AI. Mix things up.
		if (exploringframes > 0) {
			--exploringframes;
			navgoalx = explorex;
			navgoaly = explorey;
		} else {
			var mindisttobug:Float = 1000;
			for (e in Main.get().internal_entities) {
				if (Std.is(e, Bug) && e != this) {
					var disttobugx:Float = e.x - x;
					var disttobugy:Float = e.y - y;
					var disttobug:Float = Math.sqrt(disttobugx * disttobugx + disttobugy * disttobugy);
					
					if (disttobug < mindisttobug) {
						mindisttobug = disttobug;
					}
				}
			}
			
			var explorechancepersecond:Float = 1 / 15;
			if (mindisttobug < 10) explorechancepersecond = 2;
			
			if (Random.chance(100 / 60) && Random.chance(100 * explorechancepersecond)) {
				// Every 15 seconds, spend a few seconds exploring.
				explorex = x + Random.float( -100, 100);
				explorey = y + Random.float( -100, 100);
				if (explorex < 16) explorex = 16;
				if (explorex > 206) explorex = 206;
				if (explorey < 16) explorey = 16;
				if (explorey > 313) explorey = 313;
				exploringframes = Random.int(80, 200);
			}
		}
		
		var navgoaldirx:Float = navgoalx - x;
		var navgoaldiry:Float = navgoaly - y;
		var navgoaldist:Float = Math.sqrt(navgoaldirx * navgoaldirx + navgoaldiry * navgoaldiry);
		
		// ====================================================
		// PATHFINDING!
		// Hardcode sensible routes around the level.
		// ====================================================
		var movegoalx:Float = navgoalx;
		var movegoaly:Float = navgoaly;
		
		if (y < 162) {
			if (movegoaly > 162) {
				// Bypass the centre platform.
				if (x > 111 && movegoalx < 160) {
					movegoalx = 160;
					if (x < 160) movegoaly = 162 - 6;
				}
				if (x <= 111 && movegoalx > 64) {
					movegoalx = 64;
					if (x > 64) movegoaly = 162 - 6;
				}
			}
		} else if (y < 188) {
			if (movegoaly < 162) {
				// Bypass the centre platform.
				if (x > 111 && movegoalx < 160) {
					movegoalx = 160;
					if (x < 160) movegoaly = 162 + 6;
				}
				if (x <= 111 && movegoalx > 64) {
					movegoalx =64;
					if (x > 64) movegoaly = 162 + 6;
				}
			} else if (movegoaly > 188) {
				// Bypass the side platforms.
				if (movegoalx < 54) {
					movegoalx = 54;
					if (x < 54) movegoaly = 188 - 6;
				}
				if (movegoalx > 171) {
					movegoalx = 171;
					if (x > 171) movegoaly = 188 - 6;
				}
			}
		} else if (y < 225) {
			if (movegoaly < 188) {
				// Bypass the side platforms.
				if (movegoalx < 54) {
					movegoalx = 54;
					if (x < 54) movegoaly = 188 + 6;
				}
				if (movegoalx > 171) {
					movegoalx = 171;
					if (x > 171) movegoaly = 188 + 6;
				}
			} else if (movegoaly > 225) {
				// Bypass the lower center platform.
				if (x > 111 && movegoalx < 160) {
					movegoalx = 160;
					if (x < 160) movegoaly = 225 - 6;
				}
				if (x <= 111 && movegoalx > 64) {
					movegoalx =64;
					if (x > 64) movegoaly = 225 - 6;
				}
			}
		} else if (y < 260) {
			if (movegoaly < 225) {
				// Bypass the lower center platform.
				if (x > 111 && movegoalx < 160) {
					movegoalx = 160;
					if (x < 160) movegoaly = 225 + 6;
				}
				if (x <= 111 && movegoalx > 64) {
					movegoalx =64;
					if (x > 64) movegoaly = 225 + 6;
				}
			} else if (movegoaly > 260) {
				// Bypass the lower side platforms.
				if (movegoalx < 54) {
					movegoalx = 54;
					if (x < 54) movegoaly = 260 - 6;
				}
				if (movegoalx > 171) {
					movegoalx = 171;
					if (x > 171) movegoaly = 260 - 6;
				}
			}
		} else {
			if (movegoaly > 260) {
				// Bypass the lower side platforms.
				if (movegoalx < 54) {
					movegoalx = 54;
					if (x < 54) movegoaly = 260 + 6;
				}
				if (movegoalx > 171) {
					movegoalx = 171;
					if (x > 171) movegoaly = 260 + 6;
				}
			}
		}
		
		// ====================================================
		// Movement and steering code.
		// ====================================================
		
		var movegoaldirx:Float = movegoalx - x;
		var movegoaldiry:Float = movegoaly - y;
		var movegoaldist:Float = Math.sqrt(movegoaldirx * movegoaldirx + movegoaldiry * movegoaldiry);
		
		// Turn towards movegoal.
		if (Math.abs(movegoaldirx) > 0 || Math.abs(movegoaldiry) > 0) {
			var movegoalangle:Float = Math.atan2(movegoaldirx, -movegoaldiry);
			
			if (angle < movegoalangle - Math.PI) angle += 2 * Math.PI;
			if (angle > movegoalangle + Math.PI) angle -= 2 * Math.PI;
			
			angle = Utils.approach(angle, movegoalangle, 0.01, 0.035);
		}
		
		// Should we move forwards? Check dotproduct
		var dotproductmin:Float = 0.0;
		if (navgoaldist > 40) dotproductmin = 0;
		else if (navgoaldist > 20) dotproductmin = 0.7;
		else dotproductmin = 0.85;
		
		if (forwardx * movegoaldirx + forwardy * movegoaldiry > dotproductmin) {
			++rundesire;
			if (rundesire >= 10) {
				rundesire = 10;
				setanim("run");
			}
		} else {
			--rundesire;
			if (rundesire <= 0) {
				rundesire = 0;
				setanim("idle");
			}
		}
		
		if (anim == "run") {
			var speed:Float = 0.8;
			x += speed * forwardx;
			y += speed * forwardy;
		}			
		
		// Run collision
		for (c in World.colliders)
		{
			var left:Float = x - 4;
			var right:Float = x + 4;
			var top:Float = y - 4;
			var bottom:Float = y + 4;
			
			if (c.right > left && c.left < right && c.top < bottom && c.bottom > top)
			{
				// Avoidance steering.
				if (x < 60 && forwardy < 0) {
					angle += 0.05;
				} else if (x > 162 && forwardy < 0) {
					angle -= 0.05;
				} else if (x < 60 && forwardy > 0) {
					angle -= 0.05;
				} else if (x > 162 && forwardy > 0) {
					angle += 0.05;
				} else if (favoursleft) {
					angle -= 0.05;
				} else {
					angle += 0.05;
				}
				
				var leftsat:Float = c.right - left;
				var rightsat:Float = right - c.left;
				var topsat:Float = c.bottom - top;
				var bottomsat:Float = bottom - c.top;
				
				if (leftsat < rightsat && leftsat < topsat && leftsat < bottomsat) {
					x += leftsat;
					if (vx < 0) vx = 0;
				} else if (rightsat < topsat && rightsat < bottomsat) {
					x -= rightsat;
					if (vx > 0) vx = 0;
				} else if (topsat < bottomsat) {
					y += topsat;
					if (vy < 0) vy = 0;
				} else {
					y -= bottomsat;
					if (vy > 0) vy = 0;
				}
			}
		}		
	}
	
	public override function render(_camx:Float, _camy:Float)
	{
		Gfx.rotation(angle * 180 / Math.PI, Gfx.CENTER, Gfx.CENTER);

		var animframecount:Int = 1;
		var tile:Int = -1;
		
		switch (anim) {
			case "idle":
				animframecount = 1;
				tile = 0;
				
			case "run":
				animframecount = 30;
				if (animframe < 8) tile = 4;
				else if (animframe < 15) tile = 5;
				else if (animframe < 23) tile = 6;
				else tile = 7;
				
			case "infest":
				animframecount = 40;
				if (animframecount < 20) tile = 8;
				else tile = 9;
		}
		
		if (animframe >= animframecount - 1) {
			animframe -= animframe;
		}
		
		if (tile == -1) {
			trace("bug: drawing invalid tile in anim " + anim + ", frame " + animframe);
		}
		
		Gfx.drawtile(Math.round(x - _camx - 8), Math.round(y - _camy - 8), "bugs/bugsheet", tile);
		
		Gfx.reset();
	}
}
