package;

import haxegon.*;

class GameSequence 
{
	public static var frame:Int;
	
	public static var wave:Int;
	public static var wavesclear:Array<Bool>;
	
	public static var wave_maxbugpop:Int;
	public static var wave_bugsremaining:Int;
	public static var wave_poptimerreset:Int;
	
	public static var wave_poptimer:Int;

	public static var allowbugtargetplayer:Bool;
	public static var allowbugtargetbuttons:Bool;
	public static var allowbugtargetdpad:Bool;
	public static var allowbugtargetscreen:Bool;
	public static var allowtopspawn:Bool;
	
	public static var state:String;
	public static var statetimer:Int;
	
	public static function init()
	{
		frame = 0;
		
		wave = 1;
		
		state = "waveintro_pre";
		statetimer = 120;
		
		wavesclear = [false, false, false, false, false, false, false];
	}
	
	private static function bugcount() : Int
	{
		var bugcount:Int = 0;
		for (e in Main.get().internal_entities) {
			if (Std.is(e, Bug)) {
				++bugcount;
			}
		}
		return bugcount;
	}
	
	private static function startwave(_wave:Int)
	{
		wave = _wave;
		
		if (Main.get().player.killed) {
			// Reboot everything.
			Hardware.init();
			
			Main.get().player = new Player();
			Main.get().internal_entities = [Main.get().player];
			Main.get().camera = new Camera();
		}
		
		allowbugtargetplayer = false;
		allowbugtargetdpad = false;
		allowbugtargetbuttons = false;
		allowbugtargetscreen = false;
		allowtopspawn = false;
		
		switch (wave) {
			case 1:
				wave_maxbugpop = 2;
				wave_bugsremaining = 4;
				wave_poptimerreset = 120;
				
			case 2:
				wave_maxbugpop = 2;
				wave_bugsremaining = 8;
				wave_poptimerreset = 90;
				
				allowbugtargetbuttons = true;
			case 3:
				wave_maxbugpop = 2;
				wave_bugsremaining = 12;
				wave_poptimerreset = 55;
				
				allowbugtargetbuttons = true;
				allowbugtargetdpad = true;
				
			case 4:
				wave_maxbugpop = 3;
				wave_bugsremaining = 16;
				wave_poptimerreset = 90;
				
				allowbugtargetbuttons = true;
				allowbugtargetdpad = true;
				allowbugtargetplayer = true;
				allowtopspawn = true;
				
			case 5:
				wave_maxbugpop = 3;
				wave_bugsremaining = 20;
				wave_poptimerreset = 60;
				
				allowbugtargetbuttons = true;
				allowbugtargetdpad = true;
				allowbugtargetplayer = true;
				allowbugtargetscreen = true;
				allowtopspawn = true;
				
			case 6:
				wave_maxbugpop = 4;
				wave_bugsremaining = 25;
				wave_poptimerreset = 45;
				
				allowbugtargetbuttons = true;
				allowbugtargetdpad = true;
				allowbugtargetplayer = true;
				allowbugtargetscreen = true;
				allowtopspawn = true;
		}
		
		wave_poptimer = 120;
	}
	
	public static function update()
	{
		++frame;
		
		if (statetimer > 0) {
			--statetimer;
			return;
		}

		/*
		if (wave < 1) {
			startwave(1);
			return;
		}*/
		
		if (state == "waveintro_pre")
		{
			state = "waveintro";
			Sound.play("newwave");
			statetimer = 240;
		}
		else if (state == "waveintro")
		{
			startwave(wave);
			state = "gameplay";
		}
		else if (state == "outofbattery")
		{
			state = "restartwave";
			statetimer = 30;
		}
		else if (state == "restartwave")
		{
			startwave(wave);
			state = "gameplay";
		}
		else if (state == "gameplay")
		{
			if (wave >= 1 && wave <= 6) {
				if (Main.get().player.killed) {
					state = "restartwave";
					statetimer = 30;
					return;
				}
				
				if (Hardware.batterycharge <= 0.01) {
					state = "outofbattery";
					statetimer = 150;
					Main.get().player.killed = true;
				}
				if (wavesclear[wave]) {
					if (wave < 6) {
						++wave;
						state = "waveintro_pre";
						statetimer = 120;
					} else {
						state = "complete";
					}
				} else if (!wavesclear[wave]) {
					// Game logic.
					var livebugs:Int = bugcount();

					if (livebugs == 0 && wave_bugsremaining == 0 && !Main.get().player.killed) {
						wavesclear[wave] = true;
						statetimer = 120;
					}
					
					if (livebugs < wave_maxbugpop && wave_bugsremaining > 0) {
						--wave_poptimer;
						if (wave_poptimer <= 0) {
							var spawnpoint:Int = Random.int(1, allowtopspawn ? 7 : 6);
							switch (spawnpoint) {
								case 1: Main.get().internal_entities.push(new Bug(15, 128, 90));
								case 2: Main.get().internal_entities.push(new Bug(207, 128, 270));
								case 3: Main.get().internal_entities.push(new Bug(15, 233, 90));
								case 4: Main.get().internal_entities.push(new Bug(207, 233, 270));
								case 5: Main.get().internal_entities.push(new Bug(15, 284, 90));
								case 6: Main.get().internal_entities.push(new Bug(207, 284, 270));
								case 7: Main.get().internal_entities.push(new Bug(Random.int(20, 202), 12, 180));
							}
							
							--wave_bugsremaining;
							wave_poptimer = wave_poptimerreset;
						}
					} else {
						wave_poptimer = wave_poptimerreset;
					}
				}
			}
		}
	}
	
	public static function renderui()
	{
		var bugs:Int = bugcount();
		
		if (wavesclear[wave]) {
			Gfx.drawimage(0, 0, "ui/headerbar_clear");
		} else if (wave >= 1 && wave <= 6) {
			Gfx.drawimage(0, 0, "ui/headerbar");
		} else {
			Gfx.drawimage(0, 0, "ui/headerbar_blank");
		}
		
		for (i in 0 ... 6) {
			if (i + 1 == wave && !wavesclear[wave]) {
				if (frame % 40 < 20) {
					Gfx.drawimage(29 + 9 * i, 1, "ui/headerbar_waveactive1");
				} else {
					Gfx.drawimage(29 + 9 * i, 1, "ui/headerbar_waveactive2");
				}
			} else if (i + 1 <= wave) {
				Gfx.drawimage(29 + 9 * i, 1, "ui/headerbar_wavedone");
			}
		}
		
		if (bugs > 0) {
			Gfx.imagecolor = Col.BLACK;
			Gfx.drawtile(99, 2, "ui/numbersheet", (bugs + wave_bugsremaining) - 1);
			Gfx.reset();
		}
		
		if (state == "complete") {
			Gfx.drawimage(0, 0, "ui/wavecomplete");
		}
		
		if (state == "waveintro") {
			switch (wave) {
				case 1: Gfx.drawimage(0, 0, "ui/wave1");
				case 2: Gfx.drawimage(0, 0, "ui/wave2");
				case 3: Gfx.drawimage(0, 0, "ui/wave3");
				case 4: Gfx.drawimage(0, 0, "ui/wave4");
				case 5: Gfx.drawimage(0, 0, "ui/wave5");
				case 6: Gfx.drawimage(0, 0, "ui/wave6");
			}
		}
	}
}