package;

import haxegon.*;

class Hardware 
{
	public static var PLAYER:Int = 0; // Not a real component, but wedged in here for AI targeting
	
	public static var DPAD_UP:Int = 1;
	public static var DPAD_DOWN:Int = 2;
	public static var DPAD_LEFT:Int = 3;
	public static var DPAD_RIGHT:Int = 4;
	
	public static var Z:Int = 5;
	public static var X:Int = 6;
	
	public static var REACTOR:Int = 7;
	public static var SCREEN:Int = 8;
	public static var VIDEO:Int = 9;
	public static var AUDIO:Int = 10;
	
	static var keysdown:Array<Bool>;
	static var keysjustdown:Array<Bool>;
	
	static var bugsinfesting:Array<Int>;
	static var infestationlevel:Array<Int>;
	static var infestations:Array<Array<Infestation>>;
	
	public static var batterycharge:Float = 100.0;
	
	public static function init() 
	{
		keysdown = [];
		keysjustdown = [];
		bugsinfesting = [];
		infestationlevel = [];
		infestations = [];
		
		for (i in 0 ... 16) {
			keysdown[i] = false;
			keysjustdown[i] = false;
			bugsinfesting[i] = 0;
			infestationlevel[i] = 0;
			infestations[i] = [];
		}
		
		batterycharge = 100.0;
	}
	
	public static function gettargetx(_target:Int) : Float
	{
		if (_target == Hardware.PLAYER) return Main.get().player.x;
		if (_target == Hardware.DPAD_UP) return 169;
		if (_target == Hardware.DPAD_DOWN) return 169;
		if (_target == Hardware.DPAD_LEFT) return 152;
		if (_target == Hardware.DPAD_RIGHT) return 186;
		if (_target == Hardware.Z) return 35;
		if (_target == Hardware.X) return 65;
		if (_target == Hardware.REACTOR) return Random.int(88, 88+44);
		if (_target == Hardware.SCREEN) return Random.int(51, 51+123);
			
		return 100;
	}

	public static function gettargety(_target:Int) : Float
	{
		if (_target == Hardware.PLAYER) return Main.get().player.y - 5;
		if (_target == Hardware.DPAD_UP) return 192;
		if (_target == Hardware.DPAD_DOWN) return 227;
		if (_target == Hardware.DPAD_LEFT) return 210;
		if (_target == Hardware.DPAD_RIGHT) return 210;
		if (_target == Hardware.Z) return 201;
		if (_target == Hardware.X) return 219;
		if (_target == Hardware.REACTOR) return Random.int(276, 276+17);
		if (_target == Hardware.SCREEN) return Random.int(42, 42+94);
		
		return 100;
	}
	
	static function allowinput(button:Int) : Bool
	{
		return infestations[button].length == 0;
	}
	
	static function updatebutton(component:Int, k:Key) {
		if (Input.justpressed(k))
		{
			if (allowinput(component)) {
				keysjustdown[component] = true;
				keysdown[component] = true;
			} else {
				// break some stuff
				Sound.play("buttonbroken");
				removeinfestation(component);
			}
		} else if (!Input.pressed(k)) {
			keysdown[component] = false;
		}
	}
	
	public static function update()
	{
		for (i in 0 ... 16) {
			keysjustdown[i] = false;
		}
		
		updatebutton(DPAD_UP, Key.UP);
		updatebutton(DPAD_DOWN, Key.DOWN);
		updatebutton(DPAD_LEFT, Key.LEFT);
		updatebutton(DPAD_RIGHT, Key.RIGHT);
		updatebutton(Z, Key.Z);
		updatebutton(X, Key.X);
		
		// Update infestation logic
		for (i in 0 ... 16) {
			if (bugsinfesting[i] == 0) {
				while (infestations[i].length > 0) {
					removeinfestation(i);
				}
			}
		}
		
		// Update battery
		if (infestations[REACTOR].length > 0) {
			batterycharge -= 0.17; // depletes in 10 seconds
			if (batterycharge < 0) batterycharge = 0;
		} else {
			if (batterycharge < 50) batterycharge = 50;
			batterycharge += 0.17; // recovers in 5 sec
			
			if (batterycharge > 100) batterycharge = 100;
		}
	}

	public static function keypressed(component:Int) : Bool
	{
		return keysdown[component];
	}
	
	public static function keyjustpressed(component:Int) : Bool
	{
		return keysjustdown[component];
	}
	
	public static function addbug(component:Int)
	{
		++bugsinfesting[component];
	}
	
	public static function removebug(component:Int)
	{
		--bugsinfesting[component];
	}
	
	public static function addinfestation(component:Int, source:Bug)
	{
		if (infestations[component].length < 1 + 2 * bugsinfesting[component]) {
			var compx:Float = source.targetx + Random.float( -8, 8);
			var compy:Float = source.targety + Random.float( -8, 8);
			
			infestations[component].push(new Infestation(compx, compy, component));
		}
	}
	
	public static function removeinfestation(component:Int)
	{
		if (infestations[component].length > 0) {
			infestations[component].splice(0, 1)[0].destroy();
			
			if (infestations[component].length == 0) {
				for (e in Main.get().internal_entities) {
					if (Std.is(e, Bug)) {
						var bug:Bug = cast(e, Bug);
						if (bug.infesting && bug.target == component) {
							bug.infesttimer = Random.int(120, 180);
						}
					}
				}
			}
		}
		//--bugsinfesting[component];
	}
	
	public static function drawinternal(camx:Float, camy:Float)
	{
		for (i in 0 ... 16) {
			for (inf in infestations[i]) {
				inf.update();
				inf.render(camx, camy);
			}
		}
	}
	
	public static function drawexternal(camx:Float, camy:Float)
	{
		for (i in 0 ... 16) {
			for (inf in infestations[i]) {
				if (inf.component != Hardware.SCREEN) {
					inf.renderexternal(camx, camy);
				}
			}
		}
	}
	
	public static function drawscreeninfest(camx:Float, camy:Float)
	{
		for (i in 0 ... 16) {
			for (inf in infestations[i]) {
				if (inf.component == Hardware.SCREEN) {
					inf.renderexternal(camx, camy);
				}
			}
		}
	}
}