package;

import haxegon.*;
import flash.geom.Rectangle;

class Main
{
	static var instance:Main;
	public static function get():Main
	{
		return instance;
	}
	
	var frame:Int;
	
	public var camera:Camera;
	
	public var player:Player;
	public var internal_entities:Array<Entity>;
	public var internal_effects:Array<Entity>;
	public var external_effects:Array<Entity>;
	
	function init()
	{
		instance = this;
		
		// Configure libraries
		Gfx.resizescreen(426, 240, true);
		
		// Load audio
		Sound.load("fight track");
		Music.play("fight track");
		
		Sound.load("shoot", 0.3);
		Sound.load("blast", 0.15);
		Sound.load("jump", 0.2);
		Sound.load("buttonbroken", 0.5);
		Sound.load("footstep", 0.1);
		Sound.load("kill", 0.5);
		Sound.load("death", 0.5);
		Sound.load("alarm", 0.25);
		Sound.load("newwave", 0.3);
		
		// Load assets
		Gfx.loadimage("internal/background");
		Gfx.loadimage("internal/dpad_u_press");
		Gfx.loadimage("internal/dpad_d_press");
		Gfx.loadimage("internal/dpad_l_press");
		Gfx.loadimage("internal/dpad_r_press");
		Gfx.loadimage("internal/btn_press");
		
		Gfx.loadimage("external/shell");
		Gfx.loadimage("external/shell_tint");
		Gfx.loadimage("external/dpad_u_press");
		Gfx.loadimage("external/dpad_d_press");
		Gfx.loadimage("external/dpad_l_press");
		Gfx.loadimage("external/dpad_r_press");
		Gfx.loadimage("external/btn_press");
		Gfx.loadimage("external/battery_1");
		Gfx.loadimage("external/battery_2");
		Gfx.loadimage("external/battery_3");

		Gfx.loadtiles("bugs/bugsheet", 16, 16);
		Gfx.loadtiles("bugs/infestsheet", 16, 16);
		
		Gfx.loadtiles("player/tilesheet", 16, 16);
		Gfx.loadimage("player/bullet");
		
		Gfx.loadimage("ui/headerbar");
		Gfx.loadimage("ui/headerbar_clear");
		Gfx.loadimage("ui/headerbar_blank");
		Gfx.loadimage("ui/headerbar_waveactive1");
		Gfx.loadimage("ui/headerbar_waveactive2");
		Gfx.loadimage("ui/headerbar_wavedone");
		Gfx.loadtiles("ui/numbersheet", 16, 8);
		Gfx.loadimage("ui/outofbattery");

		Gfx.loadimage("ui/wave1");
		Gfx.loadimage("ui/wave2");
		Gfx.loadimage("ui/wave3");
		Gfx.loadimage("ui/wave4");
		Gfx.loadimage("ui/wave5");
		Gfx.loadimage("ui/wave6");
		Gfx.loadimage("ui/wavecomplete");
		
		// Create our virtual screens
		Gfx.createimage("screen1", 160, 132);
		Gfx.drawtoimage("screen1");
		Gfx.clearscreen(Col.GREY);

		Gfx.createimage("screen2", 160, 132);
		Gfx.drawtoimage("screen2");
		Gfx.clearscreen(Col.GREY);
		
		Gfx.drawtoscreen();
		
		// Initialise game data
		frame = 0;
		
		Hardware.init();
		GameSequence.init();
		World.init();
		
		player = new Player();
		camera = new Camera();
		
		internal_entities = [player];
		internal_effects = [];
		
		external_effects = [];
	}

	function update()
	{
		Hardware.update();
		
		GameSequence.update();
		
		// Entities and effects.
		var idx:Int = 0;
		while (idx < internal_entities.length) {
			var ent:Entity = internal_entities[idx];
			
			ent.update();
			
			if (internal_entities.length > idx && internal_entities[idx] == ent) {
				++idx;
			}
		}

		idx = 0;
		while (idx < internal_effects.length) {
			var ent:Entity = internal_effects[idx];
			
			ent.update();
			
			if (internal_effects.length > idx && internal_effects[idx] == ent) {
				++idx;
			}
		}

		idx = 0;
		while (idx < external_effects.length) {
			var ent:Entity = external_effects[idx];
			
			ent.update();
			
			if (external_effects.length > idx && external_effects[idx] == ent) {
				++idx;
			}
		}
		
		// Camera.
		camera.update();
	}
	
	function drawinternals(_ofsx:Float, _ofsy:Float, _croprectangle:Rectangle)
	{
		for (e in internal_entities) {
			if (_croprectangle.contains(e.x, e.y)) {
				e.render( -_ofsx, -_ofsy);
			}
		}
		
		for (e in internal_effects) {
			if (_croprectangle.contains(e.x, e.y)) {
				e.render( -_ofsx, -_ofsy);
			}
		}
	}
	
	function render()
	{
		++frame;
		
		if (GameSequence.state == "restartwave") {
			Gfx.clearscreen(Col.BLACK);
			return;
		}
		
		var lastscreen:String = (frame % 2) == 1 ? "screen1" : "screen2";
		var currentscreen:String = (frame % 2) == 1 ? "screen2" : "screen1";
		
		// DRAW THE CONSOLE INTERIOR
		Gfx.drawtoimage(currentscreen);
		
		var camx:Int = camera.outputx;
		var camy:Int = camera.outputy;
		
		Gfx.drawimage(31 - camx, 23 - camy, lastscreen);
		Gfx.drawimage(-camx, -camy, "internal/background");

		if (Input.pressed(Key.UP)) Gfx.drawimage(-camx + 138, -camy + 179, "internal/dpad_u_press");
		if (Input.pressed(Key.DOWN)) Gfx.drawimage(-camx + 138, -camy + 179, "internal/dpad_d_press");
		if (Input.pressed(Key.LEFT)) Gfx.drawimage(-camx + 138, -camy + 179, "internal/dpad_l_press");
		if (Input.pressed(Key.RIGHT)) Gfx.drawimage(-camx + 138, -camy + 179, "internal/dpad_r_press");
		if (Input.pressed(Key.Z)) Gfx.drawimage(-camx + 21, -camy + 187, "internal/btn_press");
		if (Input.pressed(Key.X)) Gfx.drawimage(-camx + 51, -camy + 206, "internal/btn_press");
		
		Hardware.drawinternal(camx, camy);
		
		// Draw internal entities
		drawinternals(-camx, -camy, new Rectangle(camx - 8, camy - 8, 160 + 16, 132 + 16));

		// Draw the "UI"
		GameSequence.renderui();
		
		// DRAW THE WORLD VIEW
		Gfx.drawtoscreen();
		Gfx.clearscreen();
		
		// Draw the console itself, screen first so the shell graphic can add green transparency.
		renderconsole(213 - 111, 15, currentscreen);
		
	}
	
	function renderconsole(x:Float, y:Float, screenimage:String)
	{
		var batteryratio:Float = Math.max(0, Math.min(Hardware.batterycharge / 80, 1.0));
		var screenpower:Float = 0.25 + batteryratio;
		if (batteryratio < 0.3) {
			screenpower += ((0.3 - batteryratio) / 0.3) * Random.float(0, 0.1);
		}
		
		Gfx.fillbox(x + 31, y + 23, 160, 132, 0xa1d280);
		
		Gfx.imagecolor = 0xa1d280;
		
		if (GameSequence.state == "outofbattery") {
			Gfx.imagealpha = 0.5;
			Gfx.drawimage(x + 31, y + 23, "ui/outofbattery");
		} else {
			Gfx.imagealpha = screenpower;
			Gfx.drawimage(x + 31, y + 23, screenimage);
		}
		Gfx.resetcolor();
		Gfx.resetalpha();
		
		Gfx.drawimage(x, y, "external/shell_tint");
		
		Hardware.drawscreeninfest(-x, -y);
		
		drawinternals(x, y, new Rectangle(31 - 8, 23 - 8, 160 + 16, 132 + 16));
		Gfx.drawimage(x, y, "external/shell");
		
		if (Hardware.batterycharge >= 80) {
			Gfx.drawimage(x + 17, y + 71, "external/battery_3");
		} else if (Hardware.batterycharge >= 50) {
			Gfx.drawimage(x + 17, y + 71, "external/battery_2");
		} else if (Hardware.batterycharge >= 0 && (frame % 30) < 15 && GameSequence.state != "outofbattery") {
			Gfx.drawimage(x + 17, y + 71, "external/battery_1");
			if (frame % 30 == 0) {
				Sound.play("alarm");
			}
		}
		
		if (Input.pressed(Key.UP)) Gfx.drawimage(x + 138, y + 179, "external/dpad_u_press");
		if (Input.pressed(Key.DOWN)) Gfx.drawimage(x + 138, y + 179, "external/dpad_d_press");
		if (Input.pressed(Key.LEFT)) Gfx.drawimage(x + 138, y + 179, "external/dpad_l_press");
		if (Input.pressed(Key.RIGHT)) Gfx.drawimage(x + 138, y + 179, "external/dpad_r_press");
		if (Input.pressed(Key.Z)) Gfx.drawimage(x + 21, y + 187, "external/btn_press");
		if (Input.pressed(Key.X)) Gfx.drawimage(x + 51, y + 206, "external/btn_press");
		
		Hardware.drawexternal( -x, -y);
		
		for (e in external_effects) {
			e.render(-x, -y);
		}
	}
}
