package;

import haxegon.*;

class Particle extends Entity
{
	public static var TYPE_DIRT:Int = 0;
	public static var TYPE_BLAST:Int = 1;
	public static var TYPE_SMOKE:Int = 2;
	public static var TYPE_EXTERNALMUCK:Int = 3;
	public static var TYPE_EXTERNALMUCK2:Int = 4;
	
	var radius:Float;
	var type:Int;
	var age:Int;
	var lifetime:Int;
	
	public static function gunblast(_x:Float, _y:Float)
	{
		Main.get().internal_effects.push(new Particle(_x, _y, Main.get().player.vx, Main.get().player.vy, 6, TYPE_BLAST));
	}
	
	public static function dirt(_x:Float, _y:Float)
	{
		Main.get().internal_effects.push(new Particle(_x + Random.float(-1, 1), _y + Random.float(-1, 0), Random.float(-0.05, 0.05), Random.float(-0.15, -0.05), Random.float(2, 3), TYPE_DIRT));
	}
	
	public static function explosion(_x:Float, _y:Float)
	{
		for (i in 0 ... 4) {
			var driftangle:Float = Random.float(0, 2 * Math.PI);
			var driftspeed:Float = Random.float(0.3, 1.0);
			var driftx:Float = driftspeed * Math.sin(driftangle);
			var drifty:Float = driftspeed * Math.cos(driftangle);
			
			var smokex:Float = _x + 3 * driftx;
			var smokey:Float = _y + 3 * drifty;
			Main.get().internal_effects.push(new Particle(smokex, smokey, driftx/20, drifty/20, Random.float(1.5, 3), TYPE_SMOKE));
		}
		Main.get().internal_effects.push(new Particle(_x, _y, 0, 0, 11, TYPE_BLAST));
	}

	public static function externalpurplesmoke(_x:Float, _y:Float)
	{
		for (i in 0 ... 8) {
			var driftangle:Float = Random.float(0, 2 * Math.PI);
			var driftspeed:Float = Random.float(0.3, 1.0);
			var driftx:Float = driftspeed * Math.sin(driftangle);
			var drifty:Float = driftspeed * Math.cos(driftangle);
			
			var smokex:Float = _x + 6 * driftx;
			var smokey:Float = _y + 6 * drifty;
			Main.get().external_effects.push(new Particle(smokex, smokey, driftx/20, drifty/20 + 0.25, Random.float(1.5, 3), Random.bool() ? TYPE_EXTERNALMUCK : TYPE_EXTERNALMUCK2));
		}
	}
	
	public function new(_x:Float, _y:Float, _vx:Float, _vy:Float, _radius:Float, _type:Int) 
	{
		super();
		
		x = _x;
		y = _y;
		vx = _vx;
		vy = _vy;
		
		radius = _radius;
		type = _type;
		age = 0;
		
		if (_type == TYPE_DIRT) {
			lifetime = Random.int(8, 20);
		} else if (_type == TYPE_BLAST) {
			lifetime = 3;
		} else if (_type == TYPE_SMOKE) {
			lifetime = Random.int(30, 60);
		} else if (_type == TYPE_EXTERNALMUCK || _type == TYPE_EXTERNALMUCK2) {
			lifetime = Random.int(30, 60);
		}
	}
	
	public override function update()
	{
		if (age >= lifetime) {
			Main.get().internal_effects.remove(this);
			return;
		}
		
		++age;
		
		x += vx;
		y += vy;
	}
	
	public override function render(_camx:Float, _camy:Float)
	{
		if (type == TYPE_BLAST)
		{
			if (age == 1) {
				Gfx.fillcircle(x - _camx, y - _camy, radius, Col.WHITE);
			} else {
				Gfx.fillcircle(x - _camx, y - _camy, 0.8*radius, Col.BLACK);
			}
		}
		else
		{
			var currentrad:Float = radius * (1 - age / lifetime);
			
			if (type == TYPE_SMOKE) {
				Gfx.fillcircle(x - _camx, y - _camy, currentrad, Col.BLACK);
			} else if (type == TYPE_DIRT) {
				Gfx.fillcircle(x - _camx, y - _camy, currentrad, 0x323232);
			} else if (type == TYPE_EXTERNALMUCK) {
				Gfx.fillcircle(x - _camx, y - _camy, currentrad, 0x9743b3);
			} else if (type == TYPE_EXTERNALMUCK2) {
				Gfx.fillcircle(x - _camx, y - _camy, currentrad, 0x611a5f);
			}
		}
	}
}