package;

// do *not* import haxegon.Input
import haxegon.Gfx;
import haxegon.Random;
import haxegon.Sound;

class Player extends Entity
{
	public static inline var MOVESPEED:Float = 1.75;
	public static inline var LADDERCLIMBSPEED:Float = 1.0;
	public static inline var ACCEL_GROUND:Float = 0.5;
	public static inline var DECEL_GROUND:Float = 0.1;
	public static inline var ACCEL_AIR:Float = 0.1;
	public static inline var DECEL_AIR:Float = 0.05;
	public static inline var GRAVITY:Float = 0.17;
	public static inline var MAXFALLSPEED:Float = 5;

	public static inline var SHOT_KNOCKBACK:Float = 1.0;
	
	public var onground:Bool;
	public var onladder:Bool;
	public var facingright:Bool;
	public var aimingup:Bool;
	public var aimingdown:Bool;
	
	public var anim:String;
	public var animframe:Int;
	
	public var killed:Bool;

	private function setanim(_newanim:String)
	{
		if (_newanim != anim) {
			anim = _newanim;
			animframe = 0;
		}
	}
	
	public function new() 
	{
		super();
		
		x = 59;
		y = 314;
		vx = 0;
		vy = 0;
		
		onground = true;
		onladder = false;
		
		facingright = true;
		aimingup = false;
		aimingdown = false;
		
		setanim("idle");
		
		killed = false;
	}
	
	public override function update()
	{
		++animframe;
		
		if (killed) {
			// D:
			onground = false;
			onladder = false;

			vy += GRAVITY;
			
			if (onground) {
				vx = Utils.approach(vx, 0, DECEL_GROUND, 0);
			}
			
			setanim("dead");
		}
		else
		{
			var overladder:Bool = false;
			var topofladder:Bool = false;
		
			// Collision v bugs
			for (e in Main.get().internal_entities)
			{
				if (Std.is(e, Bug))
				{
					var distx:Float = e.x - x;
					var disty:Float = e.y - y;
					var dist:Float = Math.sqrt(distx * distx + disty * disty);
					
					if (disty + vy >= 0 && disty < 4) {
						// We're in the stomp zone.
						if (Math.abs(distx) < 9) {
							cast(e, Bug).killed = true;
							Particle.explosion(0.5 * (x + e.x), 0.5 * (y + e.y));
						}
					} else if (disty > -10 && disty < 4 && !cast(e, Bug).infesting) {
						// We might be getting murdered.
						if (Math.abs(distx) < 4) {
							killed = true;
							Sound.play("death");
							GameSequence.statetimer = 180;
						}
					}
				}
			}
			
			// Test vs ladders.
			for (l in World.ladders)
			{
				var left:Float = x - 4;
				var right:Float = x + 4;
				var top:Float = y - 12;
				var bottom:Float = y;
				
				if (l.right > left && l.left < right && l.bottom > top)
				{
					if (l.top < bottom) {
						overladder = true;
					}
					
					if (l.top < bottom + 2) {
						topofladder = true;
					}
				}
			}
			
			if (onladder) {
				// Ladder movement.
				if (Hardware.keypressed(Hardware.DPAD_LEFT)) {
					vx = -LADDERCLIMBSPEED;
					facingright = false;
					if (!overladder && !topofladder) {
						onladder = false;
					}
					aimingup = false;
					aimingdown = false;
				} else if (Hardware.keypressed(Hardware.DPAD_RIGHT)) {
					vx = LADDERCLIMBSPEED;
					facingright = true;
					if (!overladder && !topofladder) {
						onladder = false;
					}
					aimingup = false;
					aimingdown = false;
				} else {
					vx = 0;
				}
				
				if (Hardware.keypressed(Hardware.DPAD_UP)) {
					if (overladder) {
						vy = -LADDERCLIMBSPEED;
					} else {
						vy = 0;
					}
					aimingup = true;
					aimingdown = false;
				} else if (Hardware.keypressed(Hardware.DPAD_DOWN)) {
					vy = LADDERCLIMBSPEED;
					if (onground || (!overladder && !topofladder)) {
						onladder = false;
					}
					aimingup = false;
					aimingdown = true;
				} else {
					vy = 0;
				}
				
				if (vx == 0 && vy == 0) {
					setanim("climb_idle");
				} else {
					setanim("climb");

					if (animframe == 0 || animframe == 15) {
						Particle.dirt(x, y);
						Sound.play("footstep");
					}
				}
			} else {
				aimingup = Hardware.keypressed(Hardware.DPAD_UP);
				aimingdown = Hardware.keypressed(Hardware.DPAD_DOWN);
				
				if (onground) {
					// Ground movement.
					if (Hardware.keypressed(Hardware.DPAD_LEFT)) {
						vx = Utils.approach(vx, -MOVESPEED, ACCEL_GROUND, 0);
						facingright = false;
					} else if (Hardware.keypressed(Hardware.DPAD_RIGHT)) {
						vx = Utils.approach(vx, MOVESPEED, ACCEL_GROUND, 0);
						facingright = true;
					} else {
						vx = Utils.approach(vx, 0, DECEL_GROUND, 0);
					}
					
					if (Hardware.keypressed(Hardware.DPAD_LEFT) || Hardware.keypressed(Hardware.DPAD_RIGHT)) {
						setanim("run");
						
						if (animframe == 0) {
							Particle.dirt(x, y);
							Sound.play("footstep");
						} else if (animframe == 10) {
							Sound.play("footstep", 0, false, 0.7);
						}
					} else {
						setanim("idle");
						
						if (vx < -0.01 || vx > 0.01) {
							if (Random.chance(33)) {
								Particle.dirt(x, y);
							}
						}
					}
				} else {
					// Air movement.
					if (Hardware.keypressed(Hardware.DPAD_LEFT)) {
						vx = Utils.approach(vx, -MOVESPEED, ACCEL_AIR, 0);
						facingright = false;
					} else if (Hardware.keypressed(Hardware.DPAD_RIGHT)) {
						vx = Utils.approach(vx, MOVESPEED, ACCEL_AIR, 0);
						facingright = true;
					} else {
						vx = Utils.approach(vx, 0, DECEL_AIR, 0);
					}
					
					setanim("jump");
				}
				
				if (Hardware.keypressed(Hardware.DPAD_UP) && overladder && !Hardware.keypressed(Hardware.Z)) {
					onladder = true;
					vy = 0;
				} else {
					if (vy < -0.5 && Hardware.keypressed(Hardware.Z)) {
						vy += 0.6 * GRAVITY;
					} else if (vy < -0.5 && Hardware.keypressed(Hardware.DPAD_DOWN)) {
						vy += 1.5 * GRAVITY;
					} else {
						vy += GRAVITY;
					}
					
					if (vy > MAXFALLSPEED) {
						vy = MAXFALLSPEED;
					}
				}
			}
			
			if (Hardware.keyjustpressed(Hardware.X)) {
				var bulletdirx:Int = 0;
				var bulletdiry:Int = 0;
				
				if (aimingup) {
					bulletdiry = -1;
				} else if (aimingdown) {
					bulletdiry = 1;
				} else {
					bulletdirx = facingright ? 1 : -1;
				}

				var firex:Float = x + (facingright ? 2 : -2);
				var firey:Float = y - 6;
				if (onladder) firey -= 1;
				
				if (aimingup) firey -= 4;
				else if (aimingdown) firey += 4;
				else firex += (facingright ? 2 : -2);
				
				Main.get().internal_entities.push(new Bullet(firex, firey, bulletdirx, bulletdiry));
				
				Particle.gunblast(firex + 2 * bulletdirx, firey + 2 * bulletdiry);
				
				Sound.play("shoot");
				
				// Recoil
				if (!onladder) {
					if (bulletdirx > 0 && !Hardware.keypressed(Hardware.DPAD_RIGHT) || bulletdirx < 0 && !Hardware.keypressed(Hardware.DPAD_LEFT)) {
						vx = Utils.approach(vx, -bulletdirx * MOVESPEED, SHOT_KNOCKBACK, 0);
					}
				}
			}
			
			if ((onground || onladder) && Hardware.keyjustpressed(Hardware.Z)) {
				Sound.play("jump");
				
				if (onground) {
					vy = -3.0;
				} else {
					// Jumping off ladder/screen
					if (!Hardware.keypressed(Hardware.DPAD_DOWN)) {
						vy = -3.0;
					}
				}
				
				setanim("jump");

				for (i in 0 ... 3) {
					Particle.dirt(x, y);
				}
				
				onground = false;
				onladder = false;
			}
		}
		
		x += vx;
		y += vy;
		
		// Run collision.
		onground = false;
		
		for (c in World.colliders)
		{
			var left:Float = x - 4;
			var right:Float = x + 4;
			var top:Float = y - 12;
			var bottom:Float = y;
			
			if (c.right > left && c.left < right && c.top < bottom && c.bottom > top)
			{
				var leftsat:Float = c.right - left;
				var rightsat:Float = right - c.left;
				var topsat:Float = c.bottom - top;
				var bottomsat:Float = bottom - c.top;
				
				if (leftsat < rightsat && leftsat < topsat && leftsat < bottomsat) {
					x += leftsat;
					if (vx < 0) vx = 0;
				} else if (rightsat < topsat && rightsat < bottomsat) {
					x -= rightsat;
					if (vx > 0) vx = 0;
				} else if (topsat < bottomsat) {
					y += topsat;
					if (vy < 0) vy = 0;
				} else {
					y -= bottomsat;
					if (vy > 0) vy = 0;
					onground = true;
				}
			}
		}
	}
	
	public override function render(_camx:Float, _camy:Float)
	{
		if (!facingright) {
			Gfx.scale( -1, 1, Gfx.CENTER, Gfx.CENTER);
		}
		
		var animframecount:Int = 1;
		var tile:Int = -1;
		
		switch (anim) {
			case "idle":
				animframecount = 80;
				
				if (animframe < 20) tile = 0;
				else if (animframe < 40) tile = 1;
				else if (animframe < 60) tile = 2;
				else if (animframe < 80) tile = 3;
				
			case "run":
				animframecount = 20;
				if (animframe < 5) tile = 4;
				else if (animframe < 10) tile = 5;
				else if (animframe < 15) tile = 6;
				else if (animframe < 20) tile = 7;
				
			case "jump":
				animframecount = 1;
				if (vy < -1.5) tile = 24;
				else if (vy < 0.0) tile = 25;
				//else if (vy < 2) tile = 26;
				else tile = 27;
				
			case "climb_idle":
				animframecount = 1;
				tile = 28;
				
			case "climb":
				animframecount = 30;
				if (animframe < 15) tile = 29;
				else tile = 28;
				
			case "dead":
				animframecount = 1;
				tile = 48;
		}
		
		if (animframe >= animframecount - 1) {
			animframe -= animframecount;
		}
		
		if (tile == -1) {
			trace("player: drawing invalid tile in anim " + anim + ", frame " + animframe);
		}
		
		if (tile != -1) {
			if (aimingup) tile += 8;
			else if (aimingdown) tile += 16;
			
			Gfx.drawtile(Math.round(x - _camx - 8), Math.round(y - _camy - 16), "player/tilesheet", tile);
		}
		
		Gfx.scale(1, 1);
	}
}
